--------------------------------------------------------------------------------
-- CERN BE-CO-HT
--------------------------------------------------------------------------------
--
-- unit name:   wrtd_ref_svec_tdc_fd
--
-- description: Top entity for WRTD reference design
--
-- Top level design of the SVEC-based WRTD node, with
-- an FMC TDC in slot 1 and an FMC Fine Delay in slot 2.
--
-- This is the standard pulse-in/pulse-out WRTD node, with the FMC TDC
-- injecting pulses into the WR network in the form of WRTD messages and
-- the FMC Fine Delay converting those messages back to pulses at the
-- destination.
--
--------------------------------------------------------------------------------
-- Copyright CERN 2014-2019
--------------------------------------------------------------------------------
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 2.0 (the "License"); you may not use this file except
-- in compliance with the License. You may obtain a copy of the License at
-- http://solderpad.org/licenses/SHL-2.0.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.gencores_pkg.all;
use work.wishbone_pkg.all;
use work.wr_board_pkg.all;
use work.wr_fabric_pkg.all;
use work.axi4_pkg.all;

library unisim;
use unisim.vcomponents.all;

entity svec7_test_top is
  generic (
    g_WRPC_INITF : string  := "../../../../wr-cores/bin/wrpc/wrc_phy16.bram";
    -- Simulation-mode enable parameter. Set by default (synthesis) to 0, and
    -- changed to non-zero in the instantiation of the top level DUT in the
    -- testbench. Its purpose is to reduce some internal counters/timeouts
    -- to speed up simulations.
    g_SIMULATION : integer := 0);
  port (
    ---------------------------------------------------------------------------
    -- Clocks/resets
    ---------------------------------------------------------------------------

    -- Reset from system fpga
    rst_n_i : in std_logic;

    -- Local oscillators
    clk_20m_vcxo_i : in std_logic;      -- 20MHz VCXO clock

    clk_62m5_pllref_p_i : in std_logic;  -- 62.5 MHz PLL reference
    clk_62m5_pllref_n_i : in std_logic;

    clk_125m_gtx_n_i : in std_logic;    -- 125 MHz GTX reference
    clk_125m_gtx_p_i : in std_logic;

    -- PLL_2AFPGA pins
    clk_fpga2_p_i : in std_logic;
    clk_fpga2_n_i : in std_logic;

    clk_si57x_p_i : in std_logic;
    clk_si57x_n_i : in std_logic;


    ---------------------------------------------------------------------------
    -- VME interface
    ---------------------------------------------------------------------------

    vme_write_n_i    : in    std_logic;
    vme_sysreset_n_i : in    std_logic;
    vme_retry_oe_o   : out   std_logic;
    vme_retry_n_o    : out   std_logic;
    vme_lword_n_b    : inout std_logic;
    vme_iackout_n_o  : out   std_logic;
    vme_iackin_n_i   : in    std_logic;
    vme_iack_n_i     : in    std_logic;
    vme_gap_i        : in    std_logic;
    vme_dtack_oe_o   : out   std_logic;
    vme_dtack_n_o    : out   std_logic;
    vme_ds_n_i       : in    std_logic_vector(1 downto 0);
    vme_data_oe_n_o  : out   std_logic;
    vme_data_dir_o   : out   std_logic;
    vme_berr_o       : out   std_logic;
    vme_as_n_i       : in    std_logic;
    vme_addr_oe_n_o  : out   std_logic;
    vme_addr_dir_o   : out   std_logic;
    vme_irq_o        : out   std_logic_vector(7 downto 1);
    vme_ga_i         : in    std_logic_vector(4 downto 0);
    vme_data_b       : inout std_logic_vector(31 downto 0);
    vme_am_i         : in    std_logic_vector(5 downto 0);
    vme_addr_b       : inout std_logic_vector(31 downto 1);


    vme_noga_i   : in std_logic_vector(4 downto 0);
    vme_use_ga_i : in std_logic;

    ---------------------------------------------------------------------------
    -- SPI interfaces to DACs
    ---------------------------------------------------------------------------

    pll20dac_din_o    : out std_logic;
    pll20dac_sclk_o   : out std_logic;
    pll20dac_sync_n_o : out std_logic;
    pll25dac_din_o    : out std_logic;
    pll25dac_sclk_o   : out std_logic;
    pll25dac_sync_n_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SFP I/O for transceiver
    ---------------------------------------------------------------------------

    sfp_txp_o         : out   std_logic;
    sfp_txn_o         : out   std_logic;
    sfp_rxp_i         : in    std_logic;
    sfp_rxn_i         : in    std_logic;
    sfp_mod_def0_i    : in    std_logic;  -- sfp detect
    sfp_mod_def1_b    : inout std_logic;  -- scl
    sfp_mod_def2_b    : inout std_logic;  -- sda
    sfp_rate_select_o : out   std_logic;
    sfp_tx_fault_i    : in    std_logic;
    sfp_tx_disable_o  : out   std_logic;
    sfp_los_i         : in    std_logic;

    ---------------------------------------------------------------------------
    -- Carrier I2C EEPROM
    ---------------------------------------------------------------------------

    carrier_scl_b : inout std_logic;
    carrier_sda_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- PCB version
    ---------------------------------------------------------------------------
    pcbrev_i : in std_logic_vector(4 downto 0);

    ---------------------------------------------------------------------------
    -- Onewire interface
    ---------------------------------------------------------------------------

    onewire_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- UART
    ---------------------------------------------------------------------------

    uart_rxd_i : in  std_logic;
    uart_txd_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SPI (flash is connected to SFPGA and routed to AFPGA
    -- once the boot process is complete)
    ---------------------------------------------------------------------------

    spi_ncs_o  : out std_logic;
    spi_mosi_o : out std_logic;
    spi_miso_i : in  std_logic;

    ---------------------------------------------------------------------------
    -- Carrier front panel LEDs and IOs
    ---------------------------------------------------------------------------

    fp_led_line_oen_o : out std_logic_vector(1 downto 0);
    fp_led_line_o     : out std_logic_vector(1 downto 0);
    fp_led_column_o   : out std_logic_vector(3 downto 0);

    fp_gpio1_b      : out std_logic;    -- PPS output
    fp_gpio2_b      : out std_logic;    -- not used
    fp_gpio3_b      : in  std_logic;    -- ext 10MHz clock input
    fp_gpio4_b      : in  std_logic;    -- ext PPS input
    fp_term_en_o    : out std_logic_vector(4 downto 1);
    fp_gpio1_a2b_o  : out std_logic;
    fp_gpio2_a2b_o  : out std_logic;
    fp_gpio34_a2b_o : out std_logic;

    fp_pushbutton_n_i : in std_logic;


    ---------------------------------------------------------------------------
    -- FMC slot 1 pins
    ---------------------------------------------------------------------------

    fmc0_la_p_b : inout std_logic_vector(33 downto 0);
    fmc0_la_n_b : inout std_logic_vector(33 downto 0);

    fmc0_pg_i          : in std_logic;
    fmc0_prsnt_m2c_n_i : in std_logic;
    fmc0_clk_m2c_p_i   : in std_logic_vector(1 downto 0);
    fmc0_clk_m2c_n_i   : in std_logic_vector(1 downto 0);

    fmc0_scl_b : inout std_logic;
    fmc0_sda_b : inout std_logic;

    fmc0_tck_o : out std_logic;
    fmc0_tms_o : out std_logic;
    fmc0_tdi_o : out std_logic;
    fmc0_tdo_i : in  std_logic;

    ---------------------------------------------------------------------------
    -- FMC slot 2 pins 
    ---------------------------------------------------------------------------

    fmc1_la_p_b : inout std_logic_vector(33 downto 0);
    fmc1_la_n_b : inout std_logic_vector(33 downto 0);

    fmc1_pg_i          : in std_logic;
    fmc1_prsnt_m2c_n_i : in std_logic;
    fmc1_clk_m2c_p_i   : in std_logic_vector(1 downto 0);
    fmc1_clk_m2c_n_i   : in std_logic_vector(1 downto 0);

    fmc1_scl_b : inout std_logic;
    fmc1_sda_b : inout std_logic;

    fmc1_tck_o : out std_logic;
    fmc1_tms_o : out std_logic;
    fmc1_tdi_o : out std_logic;
    fmc1_tdo_i : in  std_logic;


    -- P2 pins

    p2_p_b : inout std_logic_vector(19 downto 0);
    p2_n_b : inout std_logic_vector(19 downto 0);

    ---------------------------------------------------------------------------
    -- DDR Controller
    ---------------------------------------------------------------------------

    ddr_a_o       : out   std_logic_vector(14 downto 0);
    ddr_ba_o      : out   std_logic_vector(2 downto 0);
    ddr_cas_n_o   : out   std_logic;
    ddr_ck_n_o    : out   std_logic;
    ddr_ck_p_o    : out   std_logic;
    ddr_cke_o     : out   std_logic;
    ddr_dq_b      : inout std_logic_vector(63 downto 0);
    ddr_ldm_o     : out   std_logic;
    ddr_dqs_n_b   : inout std_logic_vector(7 downto 0);
    ddr_dqs_p_b   : inout std_logic_vector(7 downto 0);
    ddr_odt_o     : out   std_logic;
    ddr_ras_n_o   : out   std_logic;
    ddr_reset_n_o : out   std_logic;
    ddr_rzq_b     : inout std_logic;
    ddr_dm_o      : out   std_logic_vector(7 downto 0);
    ddr_we_n_o    : out   std_logic;
    ddr_cs_n_o    : out   std_logic;

    si57x_scl_b : inout std_logic;
    si57x_sda_b : inout std_logic;

    si57x_oe_o   : out std_logic;
    si57x_tune_o : out std_logic

    );

end entity svec7_test_top;

architecture arch of svec7_test_top is

  -----------------------------------------------------------------------------
  -- Constants
  -----------------------------------------------------------------------------

  -- WRTD Node identification (WTN2)
  constant c_WRTD_NODE_ID : std_logic_vector(31 downto 0) := x"5754_4E02";

  -- Number of masters attached to the primary wishbone crossbar
  constant c_NUM_WB_MASTERS : integer := 1;

  -- Number of slaves attached to the primary wishbone crossbar
  constant c_NUM_WB_SLAVES : integer := 3;

  -- Primary Wishbone master(s) offsets
  constant c_WB_MASTER_VME : integer := 0;

  -- Primary Wishbone slave(s) offsets
  constant c_WB_SLAVE_METADATA  : integer := 0;
  constant c_WB_SLAVE_GPIO      : integer := 1;
  constant c_WB_SLAVE_CLOCK_MON : integer := 2;

  -- Convention metadata base address
  constant c_METADATA_ADDR : t_wishbone_address := x"0000_2000";

  -- Primary wishbone crossbar layout
  constant c_WB_LAYOUT_ADDR :
    t_wishbone_address_array(c_NUM_WB_SLAVES - 1 downto 0) := (
      c_WB_SLAVE_METADATA  => c_METADATA_ADDR,
      c_WB_SLAVE_GPIO      => x"0000_8000",
      c_WB_SLAVE_CLOCK_MON => x"0001_0000");

  constant c_WB_LAYOUT_MASK :
    t_wishbone_address_array(c_NUM_WB_SLAVES - 1 downto 0) := (
      c_WB_SLAVE_METADATA  => x"0003_ffc0",   --    0x40 bytes
      c_WB_SLAVE_GPIO      => x"0003_f800",
      c_WB_SLAVE_CLOCK_MON => x"0003_f800");  -- 0x20000 bytes



  -----------------------------------------------------------------------------
  -- Signals
  -----------------------------------------------------------------------------

  -- Wishbone buse(s) from masters attached to crossbar
  signal cnx_master_out : t_wishbone_master_out_array(c_NUM_WB_MASTERS-1 downto 0);
  signal cnx_master_in  : t_wishbone_master_in_array(c_NUM_WB_MASTERS-1 downto 0);

  -- Wishbone buse(s) to slaves attached to crossbar
  signal cnx_slave_out : t_wishbone_slave_out_array(c_NUM_WB_SLAVES-1 downto 0);
  signal cnx_slave_in  : t_wishbone_slave_in_array(c_NUM_WB_SLAVES-1 downto 0);

  -- clock and reset
  signal areset_n       : std_logic;
  signal clk_sys_62m5   : std_logic;
  signal rst_sys_62m5_n : std_logic;
  signal clk_ref_62m5   : std_logic;
  signal clk_ext_ref    : std_logic;

  attribute keep : string;

  -- I2C EEPROM
  signal eeprom_sda_in  : std_logic;
  signal eeprom_sda_out : std_logic;
  signal eeprom_scl_in  : std_logic;
  signal eeprom_scl_out : std_logic;

  -- VME
  signal vme_data_b_out    : std_logic_vector(31 downto 0);
  signal vme_addr_b_out    : std_logic_vector(31 downto 1);
  signal vme_lword_n_b_out : std_logic;
  signal vme_data_dir_int  : std_logic;
  signal vme_addr_dir_int  : std_logic;
  signal vme_ga            : std_logic_vector(5 downto 0);
  signal vme_berr_n        : std_logic;
  signal vme_irq_n         : std_logic_vector(7 downto 1);
  signal vme_access_led    : std_logic;

  -- SFP
  signal sfp_sda_in  : std_logic;
  signal sfp_sda_out : std_logic;
  signal sfp_scl_in  : std_logic;
  signal sfp_scl_out : std_logic;

  -- LEDs and GPIO
  signal pps         : std_logic;
  signal pps_led     : std_logic;
  signal pps_ext_in  : std_logic;
  signal svec_led    : std_logic_vector(15 downto 0);
  signal wr_led_link : std_logic;
  signal wr_led_act  : std_logic;

  -- Interrupts
  signal irq_vector : std_logic_vector(5 downto 0);

  --  MT endpoints

  attribute iob        : string;
  attribute iob of pps : signal is "FORCE";

  signal ddr_axi4_out : t_axi4_full_master_in_512;
  signal ddr_axi4_in  : t_axi4_full_master_out_512;

  signal gpio_out : std_logic_vector(255 downto 0);
  signal gpio_in  : std_logic_vector(255 downto 0);
  signal gpio_oen : std_logic_vector(255 downto 0);


  signal clk_fpga2 : std_logic;
  signal clk_si57x : std_logic;

  signal fmc0_clk : std_logic_vector(1 downto 0);
  signal fmc1_clk : std_logic_vector(1 downto 0);


begin  -- architecture arch

  areset_n <= vme_sysreset_n_i and rst_n_i;

  cmp_buf1 : IBUFGDS
    generic map (
      DIFF_TERM    => true,             -- Differential Termination
      IBUF_LOW_PWR => true,  -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standard
      IOSTANDARD   => "DEFAULT")
    port map (
      O  => clk_fpga2,                  -- Buffer output
      I  => clk_fpga2_p_i,  -- Diff_p buffer input (connect directly to top-level port)
      IB => clk_fpga2_n_i  -- Diff_n buffer input (connect directly to top-level port)
      );

  cmp_buf2 : IBUFGDS
    generic map (
      DIFF_TERM    => true,             -- Differential Termination
      IBUF_LOW_PWR => true,  -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standard
      IOSTANDARD   => "DEFAULT")
    port map (
      O  => clk_si57x,                  -- Buffer output
      I  => clk_si57x_p_i,  -- Diff_p buffer input (connect directly to top-level port)
      IB => clk_si57x_n_i  -- Diff_n buffer input (connect directly to top-level port)
      );

  cmp_buf6 : IBUFGDS
    generic map (
      DIFF_TERM    => true,             -- Differential Termination
      IBUF_LOW_PWR => true,  -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standard
      IOSTANDARD   => "DEFAULT")
    port map (
      O  => fmc0_clk(0),                -- Buffer output
      I  => fmc0_clk_m2c_p_i(0),  -- Diff_p buffer input (connect directly to top-level port)
      IB => fmc0_clk_m2c_n_i(0)  -- Diff_n buffer input (connect directly to top-level port)
      );

  cmp_buf3 : IBUFGDS
    generic map (
      DIFF_TERM    => true,             -- Differential Termination
      IBUF_LOW_PWR => true,  -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standard
      IOSTANDARD   => "DEFAULT")
    port map (
      O  => fmc0_clk(1),                -- Buffer output
      I  => fmc0_clk_m2c_p_i(1),  -- Diff_p buffer input (connect directly to top-level port)
      IB => fmc0_clk_m2c_n_i(1)  -- Diff_n buffer input (connect directly to top-level port)
      );


  cmp_buf4 : IBUFGDS
    generic map (
      DIFF_TERM    => true,             -- Differential Termination
      IBUF_LOW_PWR => true,  -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standard
      IOSTANDARD   => "DEFAULT")
    port map (
      O  => fmc1_clk(0),                -- Buffer output
      I  => fmc1_clk_m2c_p_i(0),  -- Diff_p buffer input (connect directly to top-level port)
      IB => fmc1_clk_m2c_n_i(0)  -- Diff_n buffer input (connect directly to top-level port)
      );


  cmp_buf5 : IBUFGDS
    generic map (
      DIFF_TERM    => true,             -- Differential Termination
      IBUF_LOW_PWR => true,  -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standard
      IOSTANDARD   => "DEFAULT")
    port map (
      O  => fmc1_clk(1),                -- Buffer output
      I  => fmc1_clk_m2c_p_i(1),  -- Diff_p buffer input (connect directly to top-level port)
      IB => fmc1_clk_m2c_n_i(1)  -- Diff_n buffer input (connect directly to top-level port)
      );


  cmp_xwb_metadata : entity work.xwb_metadata
    generic map (
      g_VENDOR_ID    => x"0000_10DC",
      g_DEVICE_ID    => c_WRTD_NODE_ID,
      g_VERSION      => x"0100_0000",
      g_CAPABILITIES => x"0000_0000",
      g_COMMIT_ID    => (others => '0'))
    port map (
      clk_i   => clk_sys_62m5,
      rst_n_i => rst_sys_62m5_n,
      wb_i    => cnx_slave_in(c_WB_SLAVE_METADATA),
      wb_o    => cnx_slave_out(c_WB_SLAVE_METADATA));

  inst_svec_base : entity work.svec7_base_wr
    generic map (
      g_WITH_VIC     => true,
      g_WITH_ONEWIRE => false,
      g_WITH_SPI     => true,
      g_WITH_WR      => true,
      g_WITH_DDR     => true,
      g_APP_OFFSET   => c_METADATA_ADDR,
      g_NUM_USER_IRQ => 6,
      g_DPRAM_INITF  => g_WRPC_INITF,
      g_AUX_CLKS     => 2,
      g_FABRIC_IFACE => plain,
      g_SIMULATION   => g_SIMULATION,
      g_VERBOSE      => false)
    port map (
      rst_n_i             => areset_n,
      clk_62m5_pllref_p_i => clk_62m5_pllref_p_i,
      clk_62m5_pllref_n_i => clk_62m5_pllref_n_i,
      clk_20m_vcxo_i      => clk_20m_vcxo_i,
      clk_125m_gtx_n_i    => clk_125m_gtx_n_i,
      clk_125m_gtx_p_i    => clk_125m_gtx_p_i,
      clk_10m_ext_i       => clk_ext_ref,
      pps_ext_i           => pps_ext_in,
      vme_write_n_i       => vme_write_n_i,
      vme_sysreset_n_i    => vme_sysreset_n_i,
      vme_retry_oe_o      => vme_retry_oe_o,
      vme_retry_n_o       => vme_retry_n_o,
      vme_lword_n_b       => vme_lword_n_b,
      vme_iackout_n_o     => vme_iackout_n_o,
      vme_iackin_n_i      => vme_iackin_n_i,
      vme_iack_n_i        => vme_iack_n_i,
      vme_gap_i           => vme_gap_i,
      vme_dtack_oe_o      => vme_dtack_oe_o,
      vme_dtack_n_o       => vme_dtack_n_o,
      vme_ds_n_i          => vme_ds_n_i,
      vme_data_oe_n_o     => vme_data_oe_n_o,
      vme_data_dir_o      => vme_data_dir_o,
      vme_berr_o          => vme_berr_o,
      vme_as_n_i          => vme_as_n_i,
      vme_addr_oe_n_o     => vme_addr_oe_n_o,
      vme_addr_dir_o      => vme_addr_dir_o,
      vme_irq_o           => vme_irq_o,
      vme_ga_i            => vme_ga_i,
      vme_data_b          => vme_data_b,
      vme_am_i            => vme_am_i,
      vme_addr_b          => vme_addr_b,
      fmc0_scl_b          => fmc0_scl_b,
      fmc0_sda_b          => fmc0_sda_b,
      fmc1_scl_b          => fmc1_scl_b,
      fmc1_sda_b          => fmc1_sda_b,
      fmc0_prsnt_m2c_n_i  => fmc0_prsnt_m2c_n_i,
      fmc1_prsnt_m2c_n_i  => fmc1_prsnt_m2c_n_i,
      onewire_b           => onewire_b,
      carrier_scl_b       => carrier_scl_b,
      carrier_sda_b       => carrier_sda_b,
      spi_ncs_o           => spi_ncs_o,
      spi_mosi_o          => spi_mosi_o,
      spi_miso_i          => spi_miso_i,
      uart_rxd_i          => uart_rxd_i,
      uart_txd_o          => uart_txd_o,
      plldac_sclk_o       => pll20dac_sclk_o,
      plldac_din_o        => pll20dac_din_o,
      pll20dac_din_o      => pll20dac_din_o,
      pll20dac_sclk_o     => pll20dac_sclk_o,
      pll20dac_sync_n_o   => pll20dac_sync_n_o,
      pll25dac_din_o      => pll25dac_din_o,
      pll25dac_sclk_o     => pll25dac_sclk_o,
      pll25dac_sync_n_o   => pll25dac_sync_n_o,
      sfp_txp_o           => sfp_txp_o,
      sfp_txn_o           => sfp_txn_o,
      sfp_rxp_i           => sfp_rxp_i,
      sfp_rxn_i           => sfp_rxn_i,
      sfp_mod_def0_i      => sfp_mod_def0_i,
      sfp_mod_def1_b      => sfp_mod_def1_b,
      sfp_mod_def2_b      => sfp_mod_def2_b,
      sfp_rate_select_o   => sfp_rate_select_o,
      sfp_tx_fault_i      => sfp_tx_fault_i,
      sfp_tx_disable_o    => sfp_tx_disable_o,
      sfp_los_i           => sfp_los_i,
      pcbrev_i            => pcbrev_i,
      clk_sys_62m5_o      => clk_sys_62m5,
      rst_sys_62m5_n_o    => rst_sys_62m5_n,
      clk_ref_62m5_o      => clk_ref_62m5,
      rst_ref_62m5_n_o    => open,
      irq_user_i          => irq_vector,
      pps_p_o             => pps,
      pps_led_o           => pps_led,
      link_ok_o           => open,
      led_link_o          => wr_led_link,
      led_act_o           => wr_led_act,
      app_wb_o            => cnx_master_out(c_WB_MASTER_VME),
      app_wb_i            => cnx_master_in(c_WB_MASTER_VME),
      ddr_clk_i           => clk_sys_62m5,
      ddr_axi4_i          => ddr_axi4_in,
      ddr_axi4_o          => ddr_axi4_out
      );

  -----------------------------------------------------------------------------
  -- Primary wishbone Crossbar
  -----------------------------------------------------------------------------

  cmp_sdb_crossbar : xwb_crossbar
    generic map (
      g_VERBOSE     => false,
      g_NUM_MASTERS => c_NUM_WB_MASTERS,
      g_NUM_SLAVES  => c_NUM_WB_SLAVES,
      g_REGISTERED  => true,
      g_ADDRESS     => c_WB_LAYOUT_ADDR,
      g_MASK        => c_WB_LAYOUT_MASK)
    port map (
      clk_sys_i => clk_sys_62m5,
      rst_n_i   => rst_sys_62m5_n,
      slave_i   => cnx_master_out,
      slave_o   => cnx_master_in,
      master_i  => cnx_slave_out,
      master_o  => cnx_slave_in);

  cmp_vme_led_extend : gc_extend_pulse
    generic map (
      g_width => 5000000)
    port map (
      clk_i      => clk_sys_62m5,
      rst_n_i    => rst_sys_62m5_n,
      pulse_i    => cnx_slave_in(c_WB_MASTER_VME).cyc,
      extended_o => vme_access_led);


  U_GPIO : xwb_gpio_port
    generic map (
      g_interface_mode         => PIPELINED,
      g_address_granularity    => BYTE,
      g_num_pins               => 256,
      g_with_builtin_tristates => false)
    port map (
      clk_sys_i  => clk_sys_62m5,
      rst_n_i    => rst_sys_62m5_n,
      slave_i    => cnx_slave_in(c_WB_SLAVE_GPIO),
      slave_o    => cnx_slave_out(c_WB_SLAVE_GPIO),
      gpio_out_o => gpio_out,
      gpio_in_i  => gpio_in,
      gpio_oen_o => gpio_oen);

  -----------------------------------------------------------------------------
  -- Carrier front panel LEDs and LEMOs
  -----------------------------------------------------------------------------

  cmp_led_controller : gc_bicolor_led_ctrl
    generic map(
      g_NB_COLUMN    => 4,
      g_NB_LINE      => 2,
      g_CLK_FREQ     => 62500000,       -- in Hz
      g_REFRESH_RATE => 250             -- in Hz
      )
    port map(
      rst_n_i => rst_sys_62m5_n,
      clk_i   => clk_sys_62m5,

      led_intensity_i => "1100100",     -- in %

      led_state_i => svec_led,

      column_o   => fp_led_column_o,
      line_o     => fp_led_line_o,
      line_oen_o => fp_led_line_oen_o);

  -- LED 4
  svec_led(1 downto 0)   <= c_LED_GREEN     when wr_led_link = '1'    else c_LED_RED;
  -- LED 3
  svec_led(3 downto 2)   <= c_LED_OFF;  --     when tm_clk_aux_locked(1) = '1' else c_LED_RED;
  -- LED 2
  svec_led(5 downto 4)   <= c_LED_OFF;  --    when tm_time_valid = '1'        else c_LED_RED;
  -- LED 1
  svec_led(7 downto 6)   <= c_LED_RED_GREEN when vme_access_led = '1' else c_LED_OFF;
  -- LED 8
  svec_led(9 downto 8)   <= c_LED_RED_GREEN when wr_led_act = '1'     else c_LED_OFF;
  -- LED 7
  svec_led(11 downto 10) <= c_LED_OFF;  --     when tm_clk_aux_locked(0) = '1' else c_LED_RED;
  -- LED 6
  svec_led(13 downto 12) <= c_LED_OFF;
  -- LED 5
  svec_led(15 downto 14) <= c_LED_GREEN     when pps_led = '1'        else c_LED_OFF;

  -- Front panel IO configuration
  fp_gpio1_b      <= pps;
  fp_gpio2_b      <= gpio_out(0);
  clk_ext_ref     <= fp_gpio3_b;
  pps_ext_in      <= fp_gpio4_b;
  fp_term_en_o    <= (others => '0');
  fp_gpio1_a2b_o  <= gpio_out(1);
  fp_gpio2_a2b_o  <= gpio_out(2);
  fp_gpio34_a2b_o <= gpio_out(3);


  gpio_in(4 downto 0) <= vme_noga_i;
  gpio_in(5)          <= vme_use_ga_i;


  gen_fmc_iobufs : for i in 0 to 33 generate
    cmp_buf_fmc0 : IOBUFDS
      generic map (
        DIFF_TERM    => true,
        IBUF_LOW_PWR => true,
        IOSTANDARD   => "DEFAULT")
      port map (
        O   => gpio_in(6 + i),
        IO  => fmc0_la_p_b(i),
        IOB => fmc0_la_n_b(i),
        I   => gpio_out(6+i),
        T   => gpio_oen(6+i));

    cmp_buf_fmc1 : IOBUFDS
      generic map (
        DIFF_TERM    => true,
        IBUF_LOW_PWR => true,
        IOSTANDARD   => "DEFAULT")
      port map (
        O   => gpio_in(34+6 + i),
        IO  => fmc1_la_p_b(i),
        IOB => fmc1_la_n_b(i),
        I   => gpio_out(34+6+i),
        T   => gpio_oen(34+6+i));


  end generate gen_fmc_iobufs;

  gen_p2_iobufs : for i in 0 to 19 generate

    cmp_buf_p2 : IOBUFDS
      generic map (
        DIFF_TERM    => true,
        IBUF_LOW_PWR => true,
        IOSTANDARD   => "DEFAULT")
      port map (
        O   => gpio_in(34+34+6 + i),
        IO  => p2_p_b(i),
        IOB => p2_n_b(i),
        I   => gpio_out(34+34+6+i),
        T   => gpio_oen(34+34+6+i));

  end generate gen_p2_iobufs;

  gpio_in(128 + 0) <= fp_pushbutton_n_i;
  gpio_in(128 + 1) <= fmc0_pg_i;

  fmc0_tck_o     <= gpio_out(128+2);
  fmc0_tms_o     <= gpio_out(128 + 3);
  fmc0_tdi_o     <= gpio_out(128 + 4);
  gpio_in(128+5) <= fmc0_tdo_i;


  gpio_in(128+6) <= fmc1_pg_i;

  fmc1_tck_o        <= gpio_out(128 + 7);
  fmc1_tms_o        <= gpio_out(128 + 8);
  fmc1_tdi_o        <= gpio_out(128 + 9);
  gpio_in(128 + 10) <= fmc1_tdo_i;


  si57x_scl_b  <= gpio_out(128 + 11);
  si57x_sda_b  <= gpio_out(128 + 12);
  si57x_oe_o   <= gpio_out(128 + 13);
  si57x_tune_o <= gpio_out(128 + 14);


  cmp_clock_mon: entity work.xwb_clock_monitor
    generic map (
      g_num_clocks => 6)
    port map (
      rst_n_i   => rst_sys_62m5_n,
      clk_sys_i => clk_sys_62m5,
      clk_in_i(0) => clk_fpga2,
      clk_in_i(1) => clk_si57x,
      clk_in_i(2) => fmc0_clk(0),
      clk_in_i(3) => fmc0_clk(1),
      clk_in_i(4) => fmc1_clk(0),
      clk_in_i(5) => fmc1_clk(1),
      
      pps_p1_i  => pps,
      slave_i   => cnx_slave_in(c_WB_SLAVE_CLOCK_MON),
      slave_o   => cnx_slave_out(c_WB_SLAVE_CLOCK_MON));

  
end architecture arch;
